try:
    unicode = unicode
except NameError:
    str = str
    unicode = str
    bytes = bytes
    basestring = (str, bytes)
else:
    str = str
    unicode = unicode
    bytes = str
    basestring = basestring


import re

from .utils import basestring

try:
    PatternType = re._pattern_type
except AttributeError:
    PatternType = re.Pattern


class Event(object):
    match_regex = re.compile('^Event: .*', re.IGNORECASE)
    parsers = {}

    @staticmethod
    def register_parser(*event_name):
        def wrapper(parser):
            for name in event_name:
                Event.parsers[name] = parser

        return wrapper

    @staticmethod
    def read(event):
        lines = event.splitlines()
        (key, value) = lines[0].split(': ', 1)
        if not key.lower() == 'event':
            raise Exception()
        name = value
        keys = {}
        for i in range(1, len(lines)):
            try:
                (key, value) = lines[i].split(': ', 1)
                if key in Event.parsers:
                    Event.parsers[key](name, keys)(key, value)
                else:
                    keys[key] = value
            except:
                pass
        return Event(name, keys)

    @staticmethod
    def match(event):
        return bool(Event.match_regex.match(event))

    def __init__(self, name, keys={}):
        self.name = name
        self.keys = keys

    def __getitem__(self, item):
        return self.keys[item]

    def __setitem__(self, key, value):
        self.keys[key] = value

    def __iter__(self):
        return iter(self.keys)

    def __str__(self):
        return 'Event : %s -> %s' % (self.name, self.keys)


class EventKeyParser(object):
    def __init__(self, event, keys):
        self.event = event
        self.keys = keys

    def __call__(self, key, value):
        raise NotImplementedError()


@Event.register_parser('ChanVariable', 'DestChanVariable', 'TargetChanVariable', 'OrigTransfererChanVariable', 
                       'SecondTransfererChanVariable', 'TransfereeChanVariable', 'TransferTargetChanVariable')
class KeyValueParser(EventKeyParser):
    def __call__(self, key, value):
        if key not in self.keys:
            self.keys[key] = {}
        k, v = value.split('=', 1)
        self.keys[key][k] = v


class EventListener(object):
    def __init__(self, on_event=None, white_list=None, black_list=[], **kwargs):
        self.white_list = [white_list] if isinstance(white_list, (basestring, PatternType)) else white_list
        self.black_list = [black_list] if isinstance(black_list, (basestring, PatternType)) else black_list
        for k in list(kwargs.keys()):
            if k.startswith('on_'):
                setattr(self, k, kwargs.pop(k))
        self.assert_attrs = kwargs
        if on_event is None:
            self.on_event = getattr(self, 'on_event', self._on_event)
        else:
            self.on_event = on_event

    def check_white_list(self, event_name):
        if self.white_list is None:
            return True
        for rule in self.white_list:
            if isinstance(rule, basestring) and event_name == rule:
                return True
            if isinstance(rule, PatternType) and rule.search(event_name) is not None:
                return True
        return False

    def check_black_list(self, event_name):
        for rule in self.black_list:
            if isinstance(rule, basestring) and event_name == rule:
                return False
            if isinstance(rule, PatternType) and rule.match(event_name) is not None:
                return False
        return True

    def check_attribute(self, rules, value):
        if isinstance(rules, (PatternType, basestring)):
            rules = [rules]
        for rule in rules:
            if isinstance(rule, basestring) and rule == value:
                return True
            if isinstance(rule, PatternType) and rule.search(value):
                return True
        return False

    def check_attributes(self, event_keys):
        for k, rule in self.assert_attrs.items():
            if k not in event_keys:
                continue
            value = event_keys[k]
            if self.check_attribute(rule, value) is False:
                return False
        return True

    def check_event_name(self, event_name):
        return self.check_white_list(event_name) and self.check_black_list(event_name)

    def check_event(self, event):
        if self.check_event_name(event.name) and self.check_attributes(event.keys):
            return True
        return False

    def __call__(self, event, **kwargs):
        if self.check_event(event):
            return self.on_event(event=event, **kwargs)
        return None

    def _on_event(self, **kwargs):
        event_name = kwargs['event'].name
        method_name = 'on_%s' % event_name
        return getattr(self, method_name, lambda *args, **ks: None)(**kwargs)



import unittest

from asterisk.ami import LoginAction, LogoffAction, SimpleAction


class AMIActionTest(unittest.TestCase):
    def compare_actions(self, a1, a2):
        a1 = str(a1).split('\r\n')
        a2 = str(a2).split('\r\n')
        self.assertEqual(a1[0], a2[0])
        self.assertSetEqual(set(a1[1:]), set(a2[1:]))

    def test_login_action(self):
        expected = '\r\n'.join([
            'Action: Login',
            'Username: username',
            'Secret: password',
        ]) + '\r\n'
        action = LoginAction('username', 'password')
        self.compare_actions(action, expected)
        self.assertEqual(action.name, 'Login')
        self.assertEqual(action.Username, 'username')
        self.assertEqual(action.Secret, 'password')
        self.assertDictEqual(action.keys, {'Username': 'username', 'Secret': 'password'})
        self.assertEqual(len(action.variables), 0)

    def test_logoff_action(self):
        expected = '\r\n'.join([
            'Action: Logoff',
        ]) + '\r\n'
        action = LogoffAction()
        self.compare_actions(action, expected)
        self.assertEqual(action.name, 'Logoff')
        self.assertEqual(len(action.keys), 0)
        self.assertEqual(len(action.variables), 0)

    def test_with_variable(self):
        expected = '\r\n'.join([
            'Action: GetVar',
            'Channel: channel-1',
            'Variable: <Variable 1>=<Value 1>',
        ]) + '\r\n'
        action = SimpleAction('GetVar', Channel='channel-1')
        action['<Variable 1>'] = '<Value 1>'
        self.compare_actions(action, expected)
        self.assertEqual(action.Channel, 'channel-1')
        self.assertEqual(action['<Variable 1>'], '<Value 1>')
        action.Channel = 'channel-2'
        self.assertEqual(action.Channel, 'channel-2')



import unittest
from asterisk import ami
from tests.settings import connection, login


class CommandTest(unittest.TestCase):
    client = None

    def setUp(self):
        self.client = ami.AMIClient(**connection)
        f = self.client.login(**login)
        f.get_response()

    def tearDown(self):
        future = self.client.logoff()
        if future is not None:
            future.get_response()
        self.client.disconnect()

    def test_command_show_channels(self):
        adapter = ami.AMIClientAdapter(self.client)
        f = adapter.Command(command='core show channels')
        self.assertIsNotNone(f.response.follows)



import time
import unittest

from asterisk.ami import AMIClient
from mock_ami import AMIMock
from tests.settings import login


class ConnectionTest(unittest.TestCase):
    def setUp(self):
        self.server = AMIMock()
        self.server_address = self.server.start()
        time.sleep(0.2)

    def tearDown(self):
        self.server.stop()

    def test_start(self):
        client = AMIClient(**dict(zip(('address', 'port'), self.server_address)))
        client.connect()
        time.sleep(0.5)
        self.assertEqual(client._ami_version, '6.6.6')
        client.login(**login)
        client.disconnect()



import unittest
from asterisk import ami
from tests.settings import connection, login


class CallbackTest(unittest.TestCase):
    client = None
    response = None

    def setUp(self):
        self.client = ami.AMIClient(**connection)
        self.client.connect()
        self.response = None

    def tearDown(self):
        future = self.client.logoff()
        if future is not None:
            future.get_response()
        self.client.disconnect()

    def callback_response(self, response):
        self.response = response

    def test_login(self):
        self.assertIsNone(self.response)
        f = self.client.login(callback=self.callback_response, **login)
        f.get_response()
        self.assertIsNotNone(self.response)

    def test_originate(self):
        f = self.client.login(**login)
        f.get_response()
        adapter = ami.AMIClientAdapter(self.client)
        self.assertIsNone(self.response)
        f = adapter.Originate(
            Channel='SIP/2010',
            Exten='*65',
            Priority=1,
            Context='from-internal',
            CallerID='python',
            Async=True,
            _callback=self.callback_response
        )
        f.get_response()
        self.assertIsNotNone(self.response)



import re
import time
import unittest

try:
    from thread import start_new_thread
except ImportError:
    from _thread import start_new_thread

from asterisk.ami import Response, FutureResponse

LINE_REGEX = re.compile(r'\r?\n')


class AMIResponseTest(unittest.TestCase):
    def test_login_response(self):
        login_response = '\r\n'.join([
            'Response: Success',
            'Message: Authentication accepted',
        ]) + '\r\n'
        self.assertTrue(Response.match(login_response))
        response = Response.read(login_response)
        self.assertFalse(response.is_error())
        self.assertFalse(response.follows)
        self.assertEqual(login_response, str(response))

    def test_login_response_fail(self):
        login_response = '\r\n'.join([
            'Response: Error',
            'Message: Authentication failed',
        ]) + '\r\n'
        self.assertTrue(Response.match(login_response))
        response = Response.read(login_response)
        self.assertTrue(response.is_error())
        self.assertFalse(response.follows)
        self.assertEqual(login_response, str(response))

    def test_goodbye_response(self):
        goodbye_response = '\r\n'.join([
            'Response: Goodbye',
            'Message: Thanks for all the fish.'
        ]) + '\r\n'
        self.assertTrue(Response.match(goodbye_response))
        response = Response.read(goodbye_response)
        self.assertFalse(response.is_error())
        self.assertFalse(response.follows)
        self.assertEqual(goodbye_response, str(response))

    def test_with_follows(self):
        follows_response = '\n'.join([
            'Response: Follows\r',
            'Channel (Context Extension Pri ) State Appl. Data',
            '0 active channel(s)',
            '--END COMMAND--',
        ]) + '\r\n'
        self.assertTrue(Response.match(follows_response))
        response = Response.read(follows_response)
        self.assertFalse(response.is_error())
        self.assertIsNotNone(response.follows)
        self.assertListEqual(LINE_REGEX.split(follows_response)[1:-1], response.follows)
        self.assertEqual(follows_response, str(response))

    def test_with_key_value_follows(self):
        """
        Action:  Command
        Command: database show AMPUSER
        """
        follows_response = '\n'.join([
            'Response: Follows\r',
            'Privilege: Command\r',
            '/AMPUSER/*/concurrency_limit                      : 4                        ',
            '/AMPUSER/2000/answermode                          : disabled                 ',
            '/AMPUSER/2000/cfringtimer                         : 0                        ',
            '/AMPUSER/2000/cidname                             : Wainer                   ',
            '/AMPUSER/2000/cidnum                              : 2000                     ',
            '/AMPUSER/2000/concurrency_limit                   : 0                        ',
            '/AMPUSER/2000/device                              : 2000                     ',
            '/AMPUSER/2000/language                            :                          ',
            '/AMPUSER/2000/noanswer                            :                          ',
            '/AMPUSER/2000/outboundcid                         :                          ',
            '/AMPUSER/2000/password                            :                          ',
            '/AMPUSER/2000/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2000/recording                           :                          ',
            '/AMPUSER/2000/recording/in/external               : dontcare                 ',
            '/AMPUSER/2000/recording/in/internal               : dontcare                 ',
            '/AMPUSER/2000/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2000/recording/out/external              : dontcare                 ',
            '/AMPUSER/2000/recording/out/internal              : dontcare                 ',
            '/AMPUSER/2000/recording/priority                  : 0                        ',
            '/AMPUSER/2000/ringtimer                           : 0                        ',
            '/AMPUSER/2000/voicemail                           : novm                     ',
            '/AMPUSER/2001/answermode                          : disabled                 ',
            '/AMPUSER/2001/cfringtimer                         : 0                        ',
            '/AMPUSER/2001/cidname                             : Wainer - Local           ',
            '/AMPUSER/2001/cidnum                              : 2001                     ',
            '/AMPUSER/2001/concurrency_limit                   : 0                        ',
            '/AMPUSER/2001/device                              : 2001                     ',
            '/AMPUSER/2001/language                            :                          ',
            '/AMPUSER/2001/noanswer                            :                          ',
            '/AMPUSER/2001/outboundcid                         :                          ',
            '/AMPUSER/2001/password                            :                          ',
            '/AMPUSER/2001/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2001/recording                           :                          ',
            '/AMPUSER/2001/recording/in/external               : always                   ',
            '/AMPUSER/2001/recording/in/internal               : always                   ',
            '/AMPUSER/2001/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2001/recording/out/external              : always                   ',
            '/AMPUSER/2001/recording/out/internal              : always                   ',
            '/AMPUSER/2001/recording/priority                  : 0                        ',
            '/AMPUSER/2001/ringtimer                           : 0                        ',
            '/AMPUSER/2001/voicemail                           : novm                     ',
            '/AMPUSER/2003/answermode                          : disabled                 ',
            '/AMPUSER/2003/cfringtimer                         : 0                        ',
            '/AMPUSER/2003/cidname                             : Teo                      ',
            '/AMPUSER/2003/cidnum                              : 2003                     ',
            '/AMPUSER/2003/concurrency_limit                   : 0                        ',
            '/AMPUSER/2003/device                              : 2003                     ',
            '/AMPUSER/2003/language                            :                          ',
            '/AMPUSER/2003/noanswer                            :                          ',
            '/AMPUSER/2003/outboundcid                         :                          ',
            '/AMPUSER/2003/password                            :                          ',
            '/AMPUSER/2003/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2003/recording                           :                          ',
            '/AMPUSER/2003/recording/in/external               : always                   ',
            '/AMPUSER/2003/recording/in/internal               : always                   ',
            '/AMPUSER/2003/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2003/recording/out/external              : always                   ',
            '/AMPUSER/2003/recording/out/internal              : always                   ',
            '/AMPUSER/2003/recording/priority                  : 0                        ',
            '/AMPUSER/2003/ringtimer                           : 0                        ',
            '/AMPUSER/2003/voicemail                           : novm                     ',
            '/AMPUSER/2004/answermode                          : disabled                 ',
            '/AMPUSER/2004/cfringtimer                         : 0                        ',
            '/AMPUSER/2004/cidname                             : Gustavo - Suporte        ',
            '/AMPUSER/2004/cidnum                              : 2004                     ',
            '/AMPUSER/2004/concurrency_limit                   : 0                        ',
            '/AMPUSER/2004/device                              : 2004                     ',
            '/AMPUSER/2004/language                            :                          ',
            '/AMPUSER/2004/noanswer                            :                          ',
            '/AMPUSER/2004/outboundcid                         :                          ',
            '/AMPUSER/2004/password                            :                          ',
            '/AMPUSER/2004/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2004/recording                           :                          ',
            '/AMPUSER/2004/recording/in/external               : always                   ',
            '/AMPUSER/2004/recording/in/internal               : always                   ',
            '/AMPUSER/2004/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2004/recording/out/external              : always                   ',
            '/AMPUSER/2004/recording/out/internal              : always                   ',
            '/AMPUSER/2004/recording/priority                  : 0                        ',
            '/AMPUSER/2004/ringtimer                           : 0                        ',
            '/AMPUSER/2004/voicemail                           : novm                     ',
            '/AMPUSER/2005/answermode                          : disabled                 ',
            '/AMPUSER/2005/cfringtimer                         : 0                        ',
            '/AMPUSER/2005/cidname                             : NoteTeo                  ',
            '/AMPUSER/2005/cidnum                              : 2005                     ',
            '/AMPUSER/2005/concurrency_limit                   : 0                        ',
            '/AMPUSER/2005/device                              : 2005                     ',
            '/AMPUSER/2005/language                            :                          ',
            '/AMPUSER/2005/noanswer                            :                          ',
            '/AMPUSER/2005/outboundcid                         :                          ',
            '/AMPUSER/2005/password                            :                          ',
            '/AMPUSER/2005/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2005/recording                           :                          ',
            '/AMPUSER/2005/recording/in/external               : always                   ',
            '/AMPUSER/2005/recording/in/internal               : always                   ',
            '/AMPUSER/2005/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2005/recording/out/external              : always                   ',
            '/AMPUSER/2005/recording/out/internal              : always                   ',
            '/AMPUSER/2005/recording/priority                  : 0                        ',
            '/AMPUSER/2005/ringtimer                           : 0                        ',
            '/AMPUSER/2005/voicemail                           : novm                     ',
            '/AMPUSER/2007/answermode                          : disabled                 ',
            '/AMPUSER/2007/cfringtimer                         : 0                        ',
            '/AMPUSER/2007/cidname                             : Carol - Financeiro       ',
            '/AMPUSER/2007/cidnum                              : 2007                     ',
            '/AMPUSER/2007/concurrency_limit                   : 0                        ',
            '/AMPUSER/2007/device                              : 2007                     ',
            '/AMPUSER/2007/language                            :                          ',
            '/AMPUSER/2007/noanswer                            :                          ',
            '/AMPUSER/2007/outboundcid                         :                          ',
            '/AMPUSER/2007/password                            :                          ',
            '/AMPUSER/2007/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2007/recording                           :                          ',
            '/AMPUSER/2007/recording/in/external               : always                   ',
            '/AMPUSER/2007/recording/in/internal               : always                   ',
            '/AMPUSER/2007/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2007/recording/out/external              : always                   ',
            '/AMPUSER/2007/recording/out/internal              : always                   ',
            '/AMPUSER/2007/recording/priority                  : 0                        ',
            '/AMPUSER/2007/ringtimer                           : 0                        ',
            '/AMPUSER/2007/voicemail                           : novm                     ',
            '/AMPUSER/2008/answermode                          : disabled                 ',
            '/AMPUSER/2008/cfringtimer                         : 0                        ',
            '/AMPUSER/2008/cidname                             : Suporte NoteHP           ',
            '/AMPUSER/2008/cidnum                              : 2008                     ',
            '/AMPUSER/2008/concurrency_limit                   : 0                        ',
            '/AMPUSER/2008/device                              : 2008                     ',
            '/AMPUSER/2008/language                            :                          ',
            '/AMPUSER/2008/noanswer                            :                          ',
            '/AMPUSER/2008/outboundcid                         :                          ',
            '/AMPUSER/2008/password                            :                          ',
            '/AMPUSER/2008/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2008/recording                           :                          ',
            '/AMPUSER/2008/recording/in/external               : always                   ',
            '/AMPUSER/2008/recording/in/internal               : always                   ',
            '/AMPUSER/2008/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2008/recording/out/external              : always                   ',
            '/AMPUSER/2008/recording/out/internal              : always                   ',
            '/AMPUSER/2008/recording/priority                  : 0                        ',
            '/AMPUSER/2008/ringtimer                           : 0                        ',
            '/AMPUSER/2008/voicemail                           : novm                     ',
            '/AMPUSER/2009/answermode                          : disabled                 ',
            '/AMPUSER/2009/cfringtimer                         : 0                        ',
            '/AMPUSER/2009/cidname                             : Fernando                 ',
            '/AMPUSER/2009/cidnum                              : 2009                     ',
            '/AMPUSER/2009/concurrency_limit                   : 0                        ',
            '/AMPUSER/2009/device                              : 2009                     ',
            '/AMPUSER/2009/language                            :                          ',
            '/AMPUSER/2009/noanswer                            :                          ',
            '/AMPUSER/2009/outboundcid                         :                          ',
            '/AMPUSER/2009/password                            :                          ',
            '/AMPUSER/2009/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2009/recording                           :                          ',
            '/AMPUSER/2009/recording/in/external               : always                   ',
            '/AMPUSER/2009/recording/in/internal               : always                   ',
            '/AMPUSER/2009/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2009/recording/out/external              : always                   ',
            '/AMPUSER/2009/recording/out/internal              : always                   ',
            '/AMPUSER/2009/recording/priority                  : 10                       ',
            '/AMPUSER/2009/ringtimer                           : 0                        ',
            '/AMPUSER/2009/voicemail                           : novm                     ',
            '/AMPUSER/2010/answermode                          : disabled                 ',
            '/AMPUSER/2010/cfringtimer                         : 0                        ',
            '/AMPUSER/2010/cidname                             : Ettore - Desenvolvimento ',
            '/AMPUSER/2010/cidnum                              : 2010                     ',
            '/AMPUSER/2010/concurrency_limit                   : 0                        ',
            '/AMPUSER/2010/device                              : 3000&2010                ',
            '/AMPUSER/2010/language                            :                          ',
            '/AMPUSER/2010/noanswer                            :                          ',
            '/AMPUSER/2010/outboundcid                         :                          ',
            '/AMPUSER/2010/password                            : 1234                     ',
            '/AMPUSER/2010/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2010/recording                           :                          ',
            '/AMPUSER/2010/recording/in/external               : always                   ',
            '/AMPUSER/2010/recording/in/internal               : always                   ',
            '/AMPUSER/2010/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2010/recording/out/external              : always                   ',
            '/AMPUSER/2010/recording/out/internal              : always                   ',
            '/AMPUSER/2010/recording/priority                  : 10                       ',
            '/AMPUSER/2010/ringtimer                           : 0                        ',
            '/AMPUSER/2010/voicemail                           : novm                     ',
            '/AMPUSER/2011/answermode                          : disabled                 ',
            '/AMPUSER/2011/cfringtimer                         : 0                        ',
            '/AMPUSER/2011/cidname                             : ATA Treinamento p1       ',
            '/AMPUSER/2011/cidnum                              : 2011                     ',
            '/AMPUSER/2011/concurrency_limit                   : 0                        ',
            '/AMPUSER/2011/device                              : 2011                     ',
            '/AMPUSER/2011/language                            :                          ',
            '/AMPUSER/2011/noanswer                            :                          ',
            '/AMPUSER/2011/outboundcid                         :                          ',
            '/AMPUSER/2011/password                            :                          ',
            '/AMPUSER/2011/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2011/recording                           :                          ',
            '/AMPUSER/2011/recording/in/external               : always                   ',
            '/AMPUSER/2011/recording/in/internal               : always                   ',
            '/AMPUSER/2011/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2011/recording/out/external              : always                   ',
            '/AMPUSER/2011/recording/out/internal              : always                   ',
            '/AMPUSER/2011/recording/priority                  : 0                        ',
            '/AMPUSER/2011/ringtimer                           : 0                        ',
            '/AMPUSER/2011/voicemail                           : novm                     ',
            '/AMPUSER/2012/answermode                          : disabled                 ',
            '/AMPUSER/2012/cfringtimer                         : 0                        ',
            '/AMPUSER/2012/cidname                             : ATA Treinamento p2       ',
            '/AMPUSER/2012/cidnum                              : 2012                     ',
            '/AMPUSER/2012/concurrency_limit                   : 0                        ',
            '/AMPUSER/2012/device                              : 2012                     ',
            '/AMPUSER/2012/language                            :                          ',
            '/AMPUSER/2012/noanswer                            :                          ',
            '/AMPUSER/2012/outboundcid                         :                          ',
            '/AMPUSER/2012/password                            :                          ',
            '/AMPUSER/2012/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2012/recording                           :                          ',
            '/AMPUSER/2012/recording/in/external               : always                   ',
            '/AMPUSER/2012/recording/in/internal               : always                   ',
            '/AMPUSER/2012/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2012/recording/out/external              : always                   ',
            '/AMPUSER/2012/recording/out/internal              : always                   ',
            '/AMPUSER/2012/recording/priority                  : 0                        ',
            '/AMPUSER/2012/ringtimer                           : 0                        ',
            '/AMPUSER/2012/voicemail                           : novm                     ',
            '/AMPUSER/2013/answermode                          : disabled                 ',
            '/AMPUSER/2013/cfringtimer                         : 0                        ',
            '/AMPUSER/2013/cidname                             : Suporte SmartPhone       ',
            '/AMPUSER/2013/cidnum                              : 2013                     ',
            '/AMPUSER/2013/concurrency_limit                   : 4                        ',
            '/AMPUSER/2013/device                              : 2013                     ',
            '/AMPUSER/2013/language                            :                          ',
            '/AMPUSER/2013/noanswer                            :                          ',
            '/AMPUSER/2013/outboundcid                         :                          ',
            '/AMPUSER/2013/password                            :                          ',
            '/AMPUSER/2013/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2013/recording                           :                          ',
            '/AMPUSER/2013/recording/in/external               : always                   ',
            '/AMPUSER/2013/recording/in/internal               : always                   ',
            '/AMPUSER/2013/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2013/recording/out/external              : always                   ',
            '/AMPUSER/2013/recording/out/internal              : always                   ',
            '/AMPUSER/2013/recording/priority                  : 0                        ',
            '/AMPUSER/2013/ringtimer                           : 0                        ',
            '/AMPUSER/2013/voicemail                           : novm                     ',
            '/AMPUSER/2014/answermode                          : disabled                 ',
            '/AMPUSER/2014/cfringtimer                         : 0                        ',
            '/AMPUSER/2014/cidname                             : Teste Dispositivo        ',
            '/AMPUSER/2014/cidnum                              : 2014                     ',
            '/AMPUSER/2014/concurrency_limit                   : 0                        ',
            '/AMPUSER/2014/device                              : 2014                     ',
            '/AMPUSER/2014/language                            :                          ',
            '/AMPUSER/2014/noanswer                            :                          ',
            '/AMPUSER/2014/outboundcid                         :                          ',
            '/AMPUSER/2014/password                            :                          ',
            '/AMPUSER/2014/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2014/recording                           :                          ',
            '/AMPUSER/2014/recording/in/external               : always                   ',
            '/AMPUSER/2014/recording/in/internal               : always                   ',
            '/AMPUSER/2014/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2014/recording/out/external              : always                   ',
            '/AMPUSER/2014/recording/out/internal              : always                   ',
            '/AMPUSER/2014/recording/priority                  : 0                        ',
            '/AMPUSER/2014/ringtimer                           : 0                        ',
            '/AMPUSER/2014/voicemail                           : novm                     ',
            '/AMPUSER/2015/answermode                          : disabled                 ',
            '/AMPUSER/2015/cfringtimer                         : 0                        ',
            '/AMPUSER/2015/cidname                             : Julio - Suporte          ',
            '/AMPUSER/2015/cidnum                              : 2015                     ',
            '/AMPUSER/2015/concurrency_limit                   : 0                        ',
            '/AMPUSER/2015/device                              : 2015                     ',
            '/AMPUSER/2015/language                            :                          ',
            '/AMPUSER/2015/noanswer                            :                          ',
            '/AMPUSER/2015/outboundcid                         :                          ',
            '/AMPUSER/2015/password                            :                          ',
            '/AMPUSER/2015/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2015/recording                           :                          ',
            '/AMPUSER/2015/recording/in/external               : always                   ',
            '/AMPUSER/2015/recording/in/internal               : always                   ',
            '/AMPUSER/2015/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2015/recording/out/external              : always                   ',
            '/AMPUSER/2015/recording/out/internal              : always                   ',
            '/AMPUSER/2015/recording/priority                  : 10                       ',
            '/AMPUSER/2015/ringtimer                           : 0                        ',
            '/AMPUSER/2015/voicemail                           : novm                     ',
            '/AMPUSER/2016/answermode                          : disabled                 ',
            '/AMPUSER/2016/cfringtimer                         : 0                        ',
            '/AMPUSER/2016/cidname                             : Suporte Movel            ',
            '/AMPUSER/2016/cidnum                              : 2016                     ',
            '/AMPUSER/2016/concurrency_limit                   : 0                        ',
            '/AMPUSER/2016/device                              : 2016                     ',
            '/AMPUSER/2016/followme/changecid                  : default                  ',
            '/AMPUSER/2016/followme/ddial                      : DIRECT                   ',
            '/AMPUSER/2016/followme/fixedcid                   :                          ',
            '/AMPUSER/2016/followme/grpconf                    : DISABLED                 ',
            '/AMPUSER/2016/followme/grplist                    : 982127932#               ',
            '/AMPUSER/2016/followme/grptime                    : 60                       ',
            '/AMPUSER/2016/followme/prering                    : 0                        ',
            '/AMPUSER/2016/language                            :                          ',
            '/AMPUSER/2016/noanswer                            :                          ',
            '/AMPUSER/2016/outboundcid                         :                          ',
            '/AMPUSER/2016/password                            :                          ',
            '/AMPUSER/2016/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2016/recording                           :                          ',
            '/AMPUSER/2016/recording/in/external               : always                   ',
            '/AMPUSER/2016/recording/in/internal               : always                   ',
            '/AMPUSER/2016/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2016/recording/out/external              : always                   ',
            '/AMPUSER/2016/recording/out/internal              : always                   ',
            '/AMPUSER/2016/recording/priority                  : 10                       ',
            '/AMPUSER/2016/ringtimer                           : 0                        ',
            '/AMPUSER/2016/voicemail                           : novm                     ',
            '/AMPUSER/2020/answermode                          : disabled                 ',
            '/AMPUSER/2020/cfringtimer                         : 0                        ',
            '/AMPUSER/2020/cidname                             : UraPrincipal IMF         ',
            '/AMPUSER/2020/cidnum                              : 2020                     ',
            '/AMPUSER/2020/concurrency_limit                   : 0                        ',
            '/AMPUSER/2020/device                              : 2020                     ',
            '/AMPUSER/2020/language                            :                          ',
            '/AMPUSER/2020/noanswer                            :                          ',
            '/AMPUSER/2020/outboundcid                         :                          ',
            '/AMPUSER/2020/password                            :                          ',
            '/AMPUSER/2020/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2020/recording                           :                          ',
            '/AMPUSER/2020/recording/in/external               : always                   ',
            '/AMPUSER/2020/recording/in/internal               : always                   ',
            '/AMPUSER/2020/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2020/recording/out/external              : always                   ',
            '/AMPUSER/2020/recording/out/internal              : always                   ',
            '/AMPUSER/2020/recording/priority                  : 10                       ',
            '/AMPUSER/2020/ringtimer                           : 0                        ',
            '/AMPUSER/2020/voicemail                           : novm                     ',
            '/AMPUSER/2021/answermode                          : disabled                 ',
            '/AMPUSER/2021/cfringtimer                         : 0                        ',
            '/AMPUSER/2021/cidname                             : UraNetwork - IMF         ',
            '/AMPUSER/2021/cidnum                              : 2021                     ',
            '/AMPUSER/2021/concurrency_limit                   : 0                        ',
            '/AMPUSER/2021/device                              : 2021                     ',
            '/AMPUSER/2021/language                            :                          ',
            '/AMPUSER/2021/noanswer                            :                          ',
            '/AMPUSER/2021/outboundcid                         :                          ',
            '/AMPUSER/2021/password                            :                          ',
            '/AMPUSER/2021/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2021/recording                           :                          ',
            '/AMPUSER/2021/recording/in/external               : always                   ',
            '/AMPUSER/2021/recording/in/internal               : always                   ',
            '/AMPUSER/2021/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2021/recording/out/external              : always                   ',
            '/AMPUSER/2021/recording/out/internal              : always                   ',
            '/AMPUSER/2021/recording/priority                  : 10                       ',
            '/AMPUSER/2021/ringtimer                           : 0                        ',
            '/AMPUSER/2021/voicemail                           : novm                     ',
            '/AMPUSER/2022/answermode                          : disabled                 ',
            '/AMPUSER/2022/cfringtimer                         : 0                        ',
            '/AMPUSER/2022/cidname                             : Rafael - IMF             ',
            '/AMPUSER/2022/cidnum                              : 2022                     ',
            '/AMPUSER/2022/concurrency_limit                   : 0                        ',
            '/AMPUSER/2022/device                              : 2022                     ',
            '/AMPUSER/2022/language                            :                          ',
            '/AMPUSER/2022/noanswer                            :                          ',
            '/AMPUSER/2022/outboundcid                         :                          ',
            '/AMPUSER/2022/password                            :                          ',
            '/AMPUSER/2022/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2022/recording                           :                          ',
            '/AMPUSER/2022/recording/in/external               : always                   ',
            '/AMPUSER/2022/recording/in/internal               : always                   ',
            '/AMPUSER/2022/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2022/recording/out/external              : always                   ',
            '/AMPUSER/2022/recording/out/internal              : always                   ',
            '/AMPUSER/2022/recording/priority                  : 0                        ',
            '/AMPUSER/2022/ringtimer                           : 0                        ',
            '/AMPUSER/2022/voicemail                           : novm                     ',
            '/AMPUSER/2023/answermode                          : disabled                 ',
            '/AMPUSER/2023/cfringtimer                         : 0                        ',
            '/AMPUSER/2023/cidname                             : Shima - IMF              ',
            '/AMPUSER/2023/cidnum                              : 2023                     ',
            '/AMPUSER/2023/concurrency_limit                   : 0                        ',
            '/AMPUSER/2023/device                              : 2023                     ',
            '/AMPUSER/2023/language                            :                          ',
            '/AMPUSER/2023/noanswer                            :                          ',
            '/AMPUSER/2023/outboundcid                         :                          ',
            '/AMPUSER/2023/password                            :                          ',
            '/AMPUSER/2023/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2023/recording                           :                          ',
            '/AMPUSER/2023/recording/in/external               : always                   ',
            '/AMPUSER/2023/recording/in/internal               : always                   ',
            '/AMPUSER/2023/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2023/recording/out/external              : always                   ',
            '/AMPUSER/2023/recording/out/internal              : always                   ',
            '/AMPUSER/2023/recording/priority                  : 10                       ',
            '/AMPUSER/2023/ringtimer                           : 0                        ',
            '/AMPUSER/2023/voicemail                           : novm                     ',
            '/AMPUSER/2098/answermode                          : disabled                 ',
            '/AMPUSER/2098/callmenum                           : 2098                     ',
            '/AMPUSER/2098/cfringtimer                         : 0                        ',
            '/AMPUSER/2098/cidname                             : VoicemailSuporte         ',
            '/AMPUSER/2098/cidnum                              : 2098                     ',
            '/AMPUSER/2098/concurrency_limit                   : 0                        ',
            '/AMPUSER/2098/device                              : 2098                     ',
            '/AMPUSER/2098/language                            :                          ',
            '/AMPUSER/2098/noanswer                            :                          ',
            '/AMPUSER/2098/outboundcid                         :                          ',
            '/AMPUSER/2098/password                            :                          ',
            '/AMPUSER/2098/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2098/recording                           :                          ',
            '/AMPUSER/2098/recording/in/external               : always                   ',
            '/AMPUSER/2098/recording/in/internal               : always                   ',
            '/AMPUSER/2098/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2098/recording/out/external              : always                   ',
            '/AMPUSER/2098/recording/out/internal              : always                   ',
            '/AMPUSER/2098/recording/priority                  : 0                        ',
            '/AMPUSER/2098/ringtimer                           : 0                        ',
            '/AMPUSER/2098/voicemail                           : default                  ',
            '/AMPUSER/2099/answermode                          : disabled                 ',
            '/AMPUSER/2099/cfringtimer                         : 0                        ',
            '/AMPUSER/2099/cidname                             : Fax Digital              ',
            '/AMPUSER/2099/cidnum                              : 2099                     ',
            '/AMPUSER/2099/concurrency_limit                   : 0                        ',
            '/AMPUSER/2099/device                              : 2099                     ',
            '/AMPUSER/2099/language                            :                          ',
            '/AMPUSER/2099/noanswer                            :                          ',
            '/AMPUSER/2099/outboundcid                         :                          ',
            '/AMPUSER/2099/password                            :                          ',
            '/AMPUSER/2099/queues/qnostate                     : usestate                 ',
            '/AMPUSER/2099/recording                           :                          ',
            '/AMPUSER/2099/recording/in/external               : dontcare                 ',
            '/AMPUSER/2099/recording/in/internal               : dontcare                 ',
            '/AMPUSER/2099/recording/ondemand                  : disabled                 ',
            '/AMPUSER/2099/recording/out/external              : dontcare                 ',
            '/AMPUSER/2099/recording/out/internal              : dontcare                 ',
            '/AMPUSER/2099/recording/priority                  : 0                        ',
            '/AMPUSER/2099/ringtimer                           : 0                        ',
            '/AMPUSER/2099/voicemail                           : novm                     ',
            '/AMPUSER/3000/answermode                          : disabled                 ',
            '/AMPUSER/3000/cfringtimer                         : 0                        ',
            '/AMPUSER/3000/cidname                             : Tecnosuper               ',
            '/AMPUSER/3000/cidnum                              : 3000                     ',
            '/AMPUSER/3000/concurrency_limit                   : 0                        ',
            '/AMPUSER/3000/language                            :                          ',
            '/AMPUSER/3000/noanswer                            :                          ',
            '/AMPUSER/3000/outboundcid                         :                          ',
            '/AMPUSER/3000/password                            :                          ',
            '/AMPUSER/3000/queues/qnostate                     : usestate                 ',
            '/AMPUSER/3000/recording                           :                          ',
            '/AMPUSER/3000/recording/in/external               : always                   ',
            '/AMPUSER/3000/recording/in/internal               : always                   ',
            '/AMPUSER/3000/recording/ondemand                  : disabled                 ',
            '/AMPUSER/3000/recording/out/external              : always                   ',
            '/AMPUSER/3000/recording/out/internal              : always                   ',
            '/AMPUSER/3000/recording/priority                  : 0                        ',
            '/AMPUSER/3000/ringtimer                           : 0                        ',
            '/AMPUSER/3000/voicemail                           : novm                     ',
            '/AMPUSER/400/answermode                           : disabled                 ',
            '/AMPUSER/400/cfringtimer                          : 0                        ',
            '/AMPUSER/400/cidname                              : Videoconf                ',
            '/AMPUSER/400/cidnum                               : 400                      ',
            '/AMPUSER/400/concurrency_limit                    : 0                        ',
            '/AMPUSER/400/device                               : 400                      ',
            '/AMPUSER/400/language                             :                          ',
            '/AMPUSER/400/noanswer                             :                          ',
            '/AMPUSER/400/outboundcid                          :                          ',
            '/AMPUSER/400/password                             :                          ',
            '/AMPUSER/400/queues/qnostate                      : usestate                 ',
            '/AMPUSER/400/recording                            :                          ',
            '/AMPUSER/400/recording/in/external                : dontcare                 ',
            '/AMPUSER/400/recording/in/internal                : dontcare                 ',
            '/AMPUSER/400/recording/ondemand                   : disabled                 ',
            '/AMPUSER/400/recording/out/external               : dontcare                 ',
            '/AMPUSER/400/recording/out/internal               : dontcare                 ',
            '/AMPUSER/400/recording/priority                   : 10                       ',
            '/AMPUSER/400/ringtimer                            : 0                        ',
            '/AMPUSER/400/voicemail                            : novm                     ',
            '468 results found.',
            '--END COMMAND--'
        ]) + '\r\n'
        self.assertTrue(Response.match(follows_response))
        response = Response.read(follows_response)
        self.assertFalse(response.is_error())
        self.assertIsNotNone(response.follows)
        self.assertListEqual(LINE_REGEX.split(follows_response)[2:-1], response.follows)
        self.assertEqual(follows_response, str(response))

    def test_with_follows_queue_show_command(self):
        """
        Action: Command
        Command: queue show
        """
        follows_response = '\n'.join([
            'Response: Follows\r',
            'Privilege: Command\r',
            '911 has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      Carol - Financeiro (Local/2007@from-queue/n from hint:2007@ext-local) (ringinuse enabled) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            '910 has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      Julio - Suporte (Local/2015@from-queue/n from hint:2015@ext-local) (ringinuse enabled) (Unavailable) has taken no calls yet',
            '      Ettore - Desenvolvimento (Local/2010@from-queue/n from hint:2010@ext-local) (ringinuse enabled) (dynamic) (Unavailable) has taken no calls yet',
            '      Gustavo - Suporte (Local/2004@from-queue/n from hint:2004@ext-local) (ringinuse enabled) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            '912 has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      Carol - Financeiro (Local/2007@from-queue/n from hint:2007@ext-local) (ringinuse enabled) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            '901 has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      NoteTeo (Local/2005@from-queue/n from hint:2005@ext-local) (ringinuse enabled) (Unavailable) has taken no calls yet',
            '      Carol - Financeiro (Local/2007@from-queue/n from hint:2007@ext-local) (ringinuse enabled) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            'default has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 0s',
            '   No Members',
            '   No Callers',
            '',
            '902 has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      Julio - Suporte (Local/2015@from-queue/n from hint:2015@ext-local) (ringinuse enabled) (dynamic) (Unavailable) has taken no calls yet',
            '      Wainer (Local/2000@from-queue/n from hint:2000@ext-local) (ringinuse enabled) (dynamic) (Unavailable) has taken no calls yet',
            '      Gustavo - Suporte (Local/2004@from-queue/n from hint:2004@ext-local) (ringinuse enabled) (dynamic) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            '903 has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      Ettore - Desenvolvimento (Local/2010@from-queue/n from hint:2010@ext-local) (ringinuse enabled) (dynamic) (paused) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            '951 has 0 calls (max unlimited) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      Wainer (Local/2000@from-queue/n from hint:2000@ext-local) (ringinuse enabled) (dynamic) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            '950 has 0 calls (max 1) in \'ringall\' strategy (0s holdtime, 0s talktime), W:0, C:0, A:0, SL:0.0% within 60s',
            '   Members: ',
            '      Ettore - Desenvolvimento (Local/2010@from-queue/n from hint:2010@ext-local) (ringinuse enabled) (dynamic) (paused) (Unavailable) has taken no calls yet',
            '   No Callers',
            '',
            '--END COMMAND--',
        ]) + '\r\n'
        self.assertTrue(Response.match(follows_response))
        response = Response.read(follows_response)
        self.assertFalse(response.is_error())
        self.assertIsNotNone(response.follows)
        self.assertListEqual(LINE_REGEX.split(follows_response)[2:-1], response.follows)
        self.assertEqual(follows_response, str(response))

    def test_event(self):
        event = '\r\n'.join([
            'Event: FullyBooted',
            'Privilege: system, all',
            'Status: Fully Booted',
        ])
        self.assertFalse(Response.match(event))
        with self.assertRaises(Exception):
            Response.read(event)


TIMEOUT = 0.01


class TestFutureResponse(unittest.TestCase):
    response = None

    def test_callback(self):
        expected = 'some response'

        def callback(response):
            self.response = response

        future = FutureResponse(callback=callback)
        future.response = expected
        self.assertEqual(future.response, expected)
        self.assertEqual(self.response, expected)

    def test_lock_timeout(self):
        future = FutureResponse(timeout=TIMEOUT)
        before = time.time()
        response = future.response
        after = time.time()
        diff = after - before
        self.assertIsNone(response)
        self.assertAlmostEqual(diff, TIMEOUT, delta=0.1)

    def test_lock(self):
        expected = 'some respone'
        future = FutureResponse()

        def runnable():
            time.sleep(TIMEOUT)
            future.response = expected

        start_new_thread(runnable, (), {})
        response = future.response
        self.assertEqual(response, expected)



import unittest
from asterisk import ami
from tests.settings import connection, login


def debug(*args, **kwargs):
    print(map(str, args), map(lambda t: map(str, t), kwargs.items()))


class AutoReconnectTest(unittest.TestCase):
    client = None
    auto_reconnect = None
    response = None

    def setUp(self):
        self.client = ami.AMIClient(**connection)
        self.auto_reconnect = ami.AutoReconnect(
            self.client,
            on_reconnect=debug,
            on_disconnect=debug,
        )

    def tearDown(self):
        future = self.client.logoff()
        if future is not None:
            future.get_response()
        self.client.disconnect()

    def callback_response(self, response):
        self.response = response

    def test_login(self):
        self.assertIsNone(self.response)
        f = self.client.login(callback=self.callback_response, **login)
        f.get_response()
        self.assertIsNotNone(self.response)
        #import time
        #time.sleep(30)



import unittest
from asterisk import ami
from tests.settings import connection, login


class LoginTest(unittest.TestCase):
    client = None

    def setUp(self):
        self.client = ami.AMIClient(**connection)
        self.client.connect()

    def tearDown(self):
        future = self.client.logoff()
        if future is not None:
            future.get_response()
        self.client.disconnect()

    def test_success_login(self):
        future = self.client.login(**login)
        self.assertIsNotNone(future.response)

    def test_error_login(self):
        future = self.client.login('user', 'pass321')
        self.assertTrue(future.response.is_error())



# -*- encoding: utf-8 -*-

import unittest
from asterisk import ami


class AMIClientEncodingTest(unittest.TestCase):
    def test_decode_utf8(self):
        client = ami.AMIClient(encoding='utf-8')
        encoded = b'\xc3\x82\xc3\xa8\xc3\xb2\xc3\xa0\xc3\xab\xc3\xa8\xc3\xa9 \xc3\x8a\xc3\xa0\xc3\xab\xc3\xa8\xc3\xad\xc3\xa8\xc3\xad'
        expected = u'Âèòàëèé Êàëèíèí'
        decoded = client._decode_pack(encoded)
        self.assertEqual(expected, decoded)

    def test_decode_latin1(self):
        client = ami.AMIClient(encoding='latin-1')
        encoded = b'\xc2\xe8\xf2\xe0\xeb\xe8\xe9 \xca\xe0\xeb\xe8\xed\xe8\xed'
        expected = u'Âèòàëèé Êàëèíèí'
        decoded = client._decode_pack(encoded)
        self.assertEqual(expected, decoded)



class Action(object):
    def __init__(self, name, keys=None, variables=None):
        self.name = name
        self.keys = keys or dict()
        self.variables = variables or dict()

    def __str__(self):
        package = 'Action: %s\r\n' % self.name
        for key in self.keys:
            package += '%s: %s\r\n' % (key, self.keys[key])
        for var in self.variables:
            package += 'Variable: %s=%s\r\n' % (var, self.variables[var])
        return package

    def __getattr__(self, item):
        return self.keys[item]

    def __setattr__(self, key, value):
        if key in ('name', 'keys', 'variables'):
            return object.__setattr__(self, key, value)
        self.keys[key] = value

    def __setitem__(self, key, value):
        self.variables[key] = value

    def __getitem__(self, item):
        return self.variables[item]


class SimpleAction(Action):
    def __init__(self, name, **kwargs):
        super(SimpleAction, self).__init__(name=name, keys=kwargs)


class LoginAction(Action):
    def __init__(self, username, secret):
        Action.__init__(self, name='Login', keys={'Username': username, 'Secret': secret})


class LogoffAction(Action):
    def __init__(self):
        Action.__init__(self, name='Logoff', keys={})



import unittest

from asterisk import ami
from tests.settings import connection


class AMIClientTest(unittest.TestCase):
    client = None
    event = None

    def setUp(self):
        self.client = ami.AMIClient(**connection)

    def build_event(self, event='SomeEvent', **kwargs):
        return ami.Event(event, kwargs)

    def test_add_event_listener(self):
        def event_listener(event, **kwargs):
            self.event = event

        self.client.add_event_listener(event_listener)
        self.client.fire_recv_event(self.build_event())
        self.assertIsNotNone(self.event)
        self.client.remove_event_listener(event_listener)
        self.assertEqual(len(self.client._event_listeners), 0)

        self.event = None
        listener = self.client.add_event_listener(event_listener, white_list='OtherEvent')
        self.client.fire_recv_event(self.build_event())
        self.assertIsNone(self.event)
        self.client.remove_event_listener(listener)
        self.assertEqual(len(self.client._event_listeners), 0)

        self.client.add_event_listener(event_listener, white_list='SomeEvent')
        self.client.fire_recv_event(self.build_event())
        self.assertIsNotNone(self.event)

    registry_event = None
    varset_event = None

    def test_add_custom_on_event(self):
        def on_varset(event, **kwargs):
            self.varset_event = event

        def on_registry(event, **kwargs):
            self.registry_event = event

        self.client.add_event_listener(on_VarSet=on_varset, on_Registry=on_registry)
        self.client.fire_recv_event(self.build_event('VarSet'))
        self.assertIsNotNone(self.varset_event)
        self.assertIsNone(self.registry_event)
        self.client.fire_recv_event(self.build_event('Registry'))
        self.assertIsNotNone(self.registry_event)



import re
import socket
import threading
from functools import partial

from .action import Action, LoginAction, LogoffAction, SimpleAction
from .event import Event, EventListener
from .response import Response, FutureResponse
from .utils import str, unicode

NOOP = lambda *args, **kwargs: None

NOOP_LISTENER = dict(
    on_action=NOOP,
    on_response=NOOP,
    on_event=NOOP,
    on_connect=NOOP,
    on_disconnect=NOOP,
    on_unknown=NOOP,
)


class AMIClientListener(object):
    methods = ['on_action', 'on_response', 'on_event', 'on_connect', 'on_disconnect', 'on_unknown']

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            if k not in self.methods:
                raise TypeError('\'%s\' is an invalid keyword argument for this function' % k)
            setattr(self, k, v)

    def on_action(self, source, action):
        raise NotImplementedError()

    def on_response(self, source, response):
        raise NotImplementedError()

    def on_event(self, source, event):
        raise NotImplementedError()

    def on_connect(self, source):
        raise NotImplementedError()

    def on_disconnect(self, source, error=None):
        raise NotImplementedError()

    def on_unknown(self, source, pack):
        raise NotImplementedError()


class AMIClient(object):
    asterisk_start_regex = re.compile('^Asterisk *Call *Manager/(?P<version>([0-9]+\.)*[0-9]+)', re.IGNORECASE)
    asterisk_line_regex = re.compile(b'\r\n', re.IGNORECASE | re.MULTILINE)
    asterisk_pack_regex = re.compile(b'\r\n\r\n', re.IGNORECASE | re.MULTILINE)

    def __init__(self, address='127.0.0.1', port=5038,
                 encoding='utf-8', encoding_errors='replace',
                 timeout=3, buffer_size=2 ** 10,
                 **kwargs):
        self._action_counter = 0
        self._futures = {}
        self._listeners = []
        self._event_listeners = []
        self._address = address
        self._buffer_size = buffer_size
        self._port = port
        self._socket = None
        self._thread = None
        self.finished = None
        self._ami_version = None
        self._timeout = timeout
        self.encoding = encoding
        self.encoding_errors = encoding_errors
        if len(kwargs) > 0:
            self.add_listener(**kwargs)

    def next_action_id(self):
        id = self._action_counter
        self._action_counter += 1
        return str(id)

    def connect(self):
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._socket.settimeout(self._timeout)
        self._socket.connect((self._address, self._port))
        self.finished = threading.Event()
        self._thread = threading.Thread(target=self.listen)
        self._thread.daemon = True
        self._thread.start()

    def _fire_on_connect(self, **kwargs):
        for listener in self._listeners:
            listener.on_connect(source=self, **kwargs)

    def _fire_on_disconnect(self, **kwargs):
        for listener in self._listeners:
            listener.on_disconnect(source=self, **kwargs)

    def _fire_on_response(self, **kwargs):
        for listener in self._listeners:
            listener.on_response(source=self, **kwargs)

    def _fire_on_action(self, **kwargs):
        for listener in self._listeners:
            listener.on_action(source=self, **kwargs)

    def _fire_on_event(self, **kwargs):
        for listener in self._listeners:
            listener.on_event(source=self, **kwargs)

    def _fire_on_unknown(self, **kwargs):
        for listener in self._listeners:
            listener.on_unknown(source=self, **kwargs)

    def disconnect(self):
        self.finished.set()
        try:
            self._socket.close()
            self._thread.join(self._timeout)
        except:
            pass

    def login(self, username, secret, callback=None):
        if self.finished is None or self.finished.is_set():
            self.connect()
        return self.send_action(LoginAction(username, secret), callback)

    def logoff(self, callback=None):
        if self.finished is None or self.finished.is_set():
            return
        return self.send_action(LogoffAction(), callback)

    def send_action(self, action, callback=None):
        if 'ActionID' not in action.keys:
            action_id = self.next_action_id()
            action.keys['ActionID'] = action_id
        else:
            action_id = action.keys['ActionID']
        future = FutureResponse(callback, self._timeout)
        self._futures[action_id] = future
        self._fire_on_action(action=action)
        self.send(action)
        return future

    def send(self, pack):
        self._socket.send(bytearray(unicode(pack) + '\r\n', self.encoding))

    def _decode_pack(self, pack):
        return pack.decode(self.encoding, errors=self.encoding_errors)

    def _next_pack(self):
        data = b''
        while not self.finished.is_set():
            recv = self._socket.recv(self._buffer_size)
            if recv == b'':
                self.finished.set()
                continue
            data += recv
            if self.asterisk_line_regex.search(data):
                (pack, data) = self.asterisk_line_regex.split(data, 1)
                yield self._decode_pack(pack)
                break
        while not self.finished.is_set():
            while self.asterisk_pack_regex.search(data):
                (pack, data) = self.asterisk_pack_regex.split(data, 1)
                yield self._decode_pack(pack)
            recv = self._socket.recv(self._buffer_size)
            if recv == b'':
                self.finished.set()
                continue
            data += recv
        self._socket.close()

    def listen(self):
        pack_generator = self._next_pack()
        asterisk_start = next(pack_generator)
        match = AMIClient.asterisk_start_regex.match(asterisk_start)
        if not match:
            raise Exception()
        self._ami_version = match.group('version')
        self._fire_on_connect()
        try:
            while not self.finished.is_set():
                pack = next(pack_generator)
                self.fire_recv_pack(pack)
            self._fire_on_disconnect(error=None)
        except Exception as ex:
            self._fire_on_disconnect(error=ex)

    def fire_recv_reponse(self, response):
        self._fire_on_response(response=response)
        if response.status.lower() == 'goodbye':
            self.finished.set()
        if 'ActionID' not in response.keys:
            return
        action_id = response.keys['ActionID']
        if action_id not in self._futures:
            return
        future = self._futures.pop(action_id)
        future.response = response

    def fire_recv_event(self, event):
        self._fire_on_event(event=event)
        for listener in self._event_listeners:
            listener(event=event, source=self)

    def fire_recv_pack(self, pack):
        if Response.match(pack):
            response = Response.read(pack)
            self.fire_recv_reponse(response)
            return
        if Event.match(pack):
            event = Event.read(pack)
            self.fire_recv_event(event)
            return
        self._fire_on_unknown(pack=pack)

    def add_listener(self, listener=None, **kwargs):
        if not listener:
            default = NOOP_LISTENER.copy()
            default.update(kwargs)
            listener = AMIClientListener(**default)
        self._listeners.append(listener)
        return listener

    def remove_listener(self, listener):
        self._listeners.remove(listener)
        return listener

    def add_event_listener(self, on_event=None, **kwargs):
        if len(kwargs) > 0 and not isinstance(on_event, EventListener):
            event_listener = EventListener(on_event=on_event, **kwargs)
        else:
            event_listener = on_event
        self._event_listeners.append(event_listener)
        return event_listener

    def remove_event_listener(self, event_listener):
        self._event_listeners.remove(event_listener)


class AMIClientAdapter(object):
    def __init__(self, ami_client):
        self._ami_client = ami_client

    def _action(self, name, _callback=None, variables={}, **kwargs):
        action = Action(name, kwargs)
        action.variables = variables
        return self._ami_client.send_action(action, _callback)

    def __getattr__(self, item):
        return partial(self._action, item)


class AutoReconnect(threading.Thread):
    def __init__(self, ami_client, delay=0.5,
                 on_disconnect=lambda *args: None, on_reconnect=lambda *args: None):
        super(AutoReconnect, self).__init__()
        self.on_reconnect = on_reconnect
        self.on_disconnect = on_disconnect
        self.delay = delay
        self.finished = None
        self._ami_client = ami_client
        self._login_args = None
        self._login = None
        self._logoff = None
        self._prepare_client()

    def _prepare_client(self):
        self._login = self._ami_client.login
        self._logoff = self._ami_client.logoff
        self._ami_client.login = self._login_wrapper
        self._ami_client.logoff = self._logoff_wrapper

    def _rollback_client(self):
        self._ami_client.login = self._login
        self._ami_client.logoff = self._logoff

    def _login_wrapper(self, *args, **kwargs):
        callback = kwargs.pop('callback', None) or (lambda *a, **k: None)

        def on_login(response, *a, **k):
            if not response.is_error():
                if self._login_args is None:
                    self.finished = threading.Event()
                    self.start()
                self._login_args = (args, kwargs)
            callback(response, *a, **k)

        kwargs['callback'] = on_login
        return self._login(*args, **kwargs)

    def _logoff_wrapper(self, *args, **kwargs):
        self.finished.set()
        self._rollback_client()
        return self._logoff(*args, **kwargs)

    def ping(self):
        try:
            f = self._ami_client.send_action(Action('Ping'))
            response = f.response
            if response is not None and not response.is_error():
                return True
            self.on_disconnect(self._ami_client, response)
        except Exception as ex:
            self.on_disconnect(self._ami_client, ex)
        return False

    def try_reconnect(self):
        try:
            f = self._login(*self._login_args[0], **self._login_args[1])
            response = f.response
            if response is not None and not response.is_error():
                self.on_reconnect(self._ami_client, response)
                return True
        except:
            pass
        return False

    def run(self):
        self.finished.wait(self.delay)
        while not self.finished.is_set():
            if not self.ping():
                self.try_reconnect()
            self.finished.wait(self.delay)

    def __del__(self):
        self._rollback_client()



import socket
import threading

from rx import Observable


class AMIMock(object):
    thread = None

    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def start(self):
        self.socket.bind((socket.gethostname(), 0,))
        self.thread = threading.Thread(target=self.run)
        self.thread.daemon = True
        self.thread.start()
        return self.socket.getsockname()

    def run(self):
        self.socket.listen(5)

        def clients_iter():
            try:
                while True:
                    yield self.socket.accept()
            except:
                pass

        def send_start(c):
            return c[0].send(b'Asterisk Call Manager/6.6.6\r\n\r\n')

        Observable.from_(clients_iter()) \
            .subscribe(send_start)

    def stop(self):
        self.socket.close()
        self.thread.join(1)



import unittest

from asterisk import ami


class AMIEventTest(unittest.TestCase):
    def test_dict(self):
        keys = {'a': 1, 'b': 2}
        event = ami.Event('TestEvent', dict(keys))
        self.assertEqual(event['a'], 1)
        self.assertEqual(event['b'], 2)
        self.assertSetEqual(set(iter(keys)), set(iter(event)))
        self.assertIn('a', event)



import re
import threading


class Response(object):
    match_regex = re.compile('^Response: .*', re.IGNORECASE)
    key_regex = re.compile('^[a-zA-Z0-9_\-]+$')

    @staticmethod
    def read(response):
        lines = response.splitlines()
        (key, value) = map(lambda s: s.strip(), lines[0].split(':', 1))
        if not key.lower() == 'response':
            raise Exception()
        status = value
        keys = {}
        follows = []
        keys_and_follows = iter(lines[1:])
        for line in keys_and_follows:
            try:
                (key, value) = line.split(':', 1)
                if not Response.key_regex.match(key):
                    raise key
                keys[key.strip()] = value.strip()
            except:
                follows.append(line)
                break
        for line in keys_and_follows:
            follows.append(line)
        return Response(status, keys, follows)

    @staticmethod
    def match(response):
        return bool(Response.match_regex.match(response))

    def __init__(self, status, keys={}, fallows=None):
        self.status = status
        self.keys = keys
        self.follows = fallows

    def __str__(self):
        package = 'Response: %s\r\n' % self.status
        for key in self.keys:
            package += '%s: %s\r\n' % (key, self.keys[key])
        if self.follows:
            package += '\n'.join(self.follows) + '\r\n'
        return package

    def is_error(self):
        return self.status.lower() == 'error'


class FutureResponse(object):
    def __init__(self, callback=None, timeout=None):
        self._timeout = timeout
        self._response = None
        self._lock = threading.Condition()
        self._callback = callback

    def set_response(self, response):
        try:
            if self._callback is not None:
                self._callback(response)
        except Exception as ex:
            traceback.print_exc()
        finally:
            self._lock.acquire()
            self._response = response
            self._lock.notifyAll()
            self._lock.release()

    def get_response(self):
        if self._response is not None:
            return self._response
        self._lock.acquire()
        self._lock.wait(self._timeout)
        self._lock.release()
        return self._response

    response = property(get_response, set_response)



# -*- encoding: utf-8 -*-
import os

from setuptools import setup, find_packages

from asterisk import __version__


def read(file_name):
    with open(os.path.join(os.path.dirname(__file__), file_name)) as f:
        return f.read()


requirements = read('requirements.txt').splitlines()
str_version = '.'.join(map(str, __version__))

setup(
    name='asterisk-ami',
    version=str_version,
    description='Python AMI Client',
    long_description=read('README.rst'),
    url='https://github.com/ettoreleandrotognoli/python-ami/',
    download_url='https://github.com/ettoreleandrotognoli/python-ami/tree/%s/' % str_version,
    license='BSD',
    author=u'Éttore Leandro Tognoli',
    author_email='ettore.leandro.tognoli@gmail.com',
    data_files=['requirements.txt'],
    packages=find_packages(exclude=['tests', 'examples']),
    include_package_data=True,
    keywords=['asterisk', 'ami'],
    classifiers=[
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: BSD License',
    ],
    install_requires=filter(None, requirements),
    # tests_require=[],
)



import unittest
import re
from random import sample
from asterisk import ami

letters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
event_name_size = 8
samples_to_test = 10


class WhiteListTest(unittest.TestCase):
    def test_none_white_list(self):
        event_listener = ami.EventListener(white_list=None)
        for i in range(samples_to_test):
            event_name = "".join(sample(letters, event_name_size))
            self.assertTrue(event_listener.check_event_name(event_name))

    def test_empty_white_list(self):
        event_listener = ami.EventListener(white_list=[])
        for i in range(samples_to_test):
            event_name = "".join(sample(letters, event_name_size))
            self.assertFalse(event_listener.check_event_name(event_name))

    def test_string_white_list(self):
        string_rules = list(set(["".join(sample(letters, event_name_size)) for i in range(samples_to_test)]))
        in_white_list = string_rules[:int(len(string_rules) / 2)]
        out_white_list = string_rules[int(len(string_rules) / 2):]
        event_listener = ami.EventListener(white_list=in_white_list)
        for rule in in_white_list:
            self.assertTrue(event_listener.check_event_name(rule))
        for rule in out_white_list:
            self.assertFalse(event_listener.check_event_name(rule))

    def test_regex_white_list(self):
        events = list(set(["".join(sample(letters, event_name_size)) for i in range(samples_to_test)]))
        all_event_listener = ami.EventListener(white_list=re.compile('.*'))
        for event_name in events:
            self.assertTrue(all_event_listener.check_event_name(event_name))
        none_event_listener = ami.EventListener(white_list=[re.compile('[0-9]+')])
        for event_name in events:
            self.assertFalse(none_event_listener.check_event_name(event_name))


class BlackListTest(unittest.TestCase):
    def test_empty_black_list(self):
        event_listener = ami.EventListener(black_list=[])
        for i in range(samples_to_test):
            event_name = sample(letters, event_name_size)
            self.assertTrue(event_listener.check_event_name(event_name))

    def test_string_black_list(self):
        string_rules = list(set(["".join(sample(letters, event_name_size)) for i in range(samples_to_test)]))
        in_black_list = string_rules[:int(len(string_rules) / 2)]
        out_black_list = string_rules[int(len(string_rules) / 2):]
        event_listener = ami.EventListener(black_list=in_black_list)
        for rule in in_black_list:
            self.assertFalse(event_listener.check_event_name(rule))
        for rule in out_black_list:
            self.assertTrue(event_listener.check_event_name(rule))

    def test_regex_black_list(self):
        events = list(set(["".join(sample(letters, event_name_size)) for i in range(samples_to_test)]))
        none_event_listener = ami.EventListener(black_list=re.compile('.*'))
        for event_name in events:
            self.assertFalse(none_event_listener.check_event_name(event_name))
        all_event_listener = ami.EventListener(black_list=[re.compile('[0-9]+')])
        for event_name in events:
            self.assertTrue(all_event_listener.check_event_name(event_name))


class EventListenerTest(unittest.TestCase):
    def build_some_event(self, **kwargs):
        return ami.Event('SomeEvent', kwargs)

    def receive_event(self, event, **kwargs):
        self.assertIsInstance(event, ami.Event)
        return True

    def test_on_event(self):
        event_listener = ami.EventListener(on_event=self.receive_event)
        result = event_listener(event=self.build_some_event(), source=self)
        self.assertTrue(result)

    def test_custom_on_event(self):
        class CustomOnEvent(ami.EventListener):
            def on_event(self, **kwargs):
                return True

        custom_on_event = CustomOnEvent()
        self.assertTrue(custom_on_event(event=self.build_some_event(), source=self))

        custom_on_event = CustomOnEvent(on_event=lambda *args, **kwargs: False)
        self.assertFalse(custom_on_event(event=self.build_some_event(), source=self))

    def test_on_some_event(self):
        event_listener = ami.EventListener(on_SomeEvent=self.receive_event)
        result = event_listener(event=self.build_some_event(), source=self)
        self.assertTrue(result)

    def test_custom_listener(self):
        class SomeEventListener(ami.EventListener):
            def on_SomeEvent(cls, **kwargs):
                return self.receive_event(**kwargs)

        event_listener = SomeEventListener()
        result = event_listener(event=self.build_some_event(), source=self)
        self.assertTrue(result)

    def test_attr_filter(self):
        event_listener = ami.EventListener(on_event=self.receive_event, Value1='teste')
        self.assertIsNone(event_listener(event=self.build_some_event(Value1='null')))
        self.assertTrue(event_listener(event=self.build_some_event(Value1='teste')))
        event_listener = ami.EventListener(on_event=self.receive_event, Value1=re.compile('^t.*'))
        self.assertIsNone(event_listener(event=self.build_some_event(Value1='null')))
        self.assertTrue(event_listener(event=self.build_some_event(Value1='teste')))

    def test_multiple_attr_filter(self):
        event_listener = ami.EventListener(on_event=self.receive_event, Value1=['teste', 'None'])
        self.assertIsNone(event_listener(event=self.build_some_event(Value1='null')))
        self.assertTrue(event_listener(event=self.build_some_event(Value1='teste')))
        self.assertTrue(event_listener(event=self.build_some_event(Value1='None')))
        event_listener = ami.EventListener(on_event=self.receive_event, Value1=[re.compile('^t.*'), re.compile('a')])
        self.assertIsNone(event_listener(event=self.build_some_event(Value1='null')))
        self.assertTrue(event_listener(event=self.build_some_event(Value1='teste')))
        self.assertTrue(event_listener(event=self.build_some_event(Value1='__a__')))


class MultipleChannelVariablesTestCase(unittest.TestCase):
    def test_event_with_multiple_channel_variables(self):
        channel_variable = {
            'VAR1': 'VAR1',
            'VAR2': 'VAR2'
        }

        dest_channel_vasiable = {
            'VAR1': 'D_VAR1',
            'VAR2': 'D_VAR2'
        }

        ami_data = '\n'.join([
            "Event: AgentCalled",
            "ChanVariable: VAR1=%(VAR1)s" % channel_variable,
            "ChanVariable: VAR2=%(VAR2)s" % channel_variable,
            "DestChanVariable: VAR1=%(VAR1)s" % dest_channel_vasiable,
            "DestChanVariable: VAR2=%(VAR2)s" % dest_channel_vasiable,
        ])

        event = ami.Event.read(ami_data)

        self.assertDictEqual(event['ChanVariable'], channel_variable)
        self.assertDictEqual(event['DestChanVariable'], dest_channel_vasiable)

    def test_event_without_channel_variables(self):
        ami_data = '\n'.join([
            "Event: AgentCalled",
            "SomeVariable: SomeValue"
        ])

        event = ami.Event.read(ami_data)

        self.assertNotIn('ChanVariable', event)
        self.assertNotIn('DestChanVariable', event)
